@@ -158,7 +158,6 @@ main(int argc, char *argv[])
     register char **ptr;
     pid_t pid;
     int client_given = 0, server_given = 0;
-    int client_args_given = 0, server_args_given = 0;
     int start_of_client_args, start_of_server_args;
     struct sigaction sa, si;
 #ifdef __APPLE__
@@ -181,7 +180,8 @@ main(int argc, char *argv[])
     }
     start_of_client_args = (cptr - client);
     while (argc && strcmp(*argv, "--")) {
-        client_args_given++;
+        if (cptr > clientargv + sizeof(clientargv) / sizeof(*clientargv) - 2)
+            Fatalx("too many client arguments");
         *cptr++ = *argv++;
         argc--;
     }
@@ -223,7 +223,8 @@ main(int argc, char *argv[])
         }
 
         /* All other server arguments are just passed through  */
-        server_args_given++;
+        if (sptr > serverargv + sizeof(serverargv) / sizeof(*serverargv) - 2)
+            Fatalx("too many server arguments");
         *sptr++ = *argv++;
     }
 
@@ -383,15 +384,16 @@ waitforserver(void)
 
         if (length < 0)
           Fatal("reading displayfd pipe failed");
-        displayfd_buf[length] = '\0';
-
-        printf("read display number '%s' from X server\n", displayfd_buf);
-        displayNum = displayfd_buf;
 
         /* write the DISPLAY received from the server to the original displayfd */
         /* XXX: this should happen after connections are being accepted */
         write(original_displayfd, displayfd_buf+1, length);
-        write(original_displayfd, "\n", 1);
+
+        /* erase trailing LF */
+        displayfd_buf[length] = '\0';
+
+        printf("read display number '%s' from X server\n", displayfd_buf);
+        displayNum = displayfd_buf;
     }
 
     for (cycles = 0; cycles < ncycles; cycles++) {
@@ -701,7 +703,7 @@ shutdown(void)
             Fatal("Unable to run program \"%s\"", "kbd_mode");
             break;
 
-        case 1:
+        case -1:
             Error("fork failed");
             break;
 
